@@ -27,13 +27,12 @@
 #include <kstandarddirs.h>
 #include <klocale.h>
 
-#include <QtCore/QMutex>
 #include <QtCore/QStringList>
 #include <QtNetwork/QSslKey>
 #include <QtNetwork/QSslCipher>
+#include <QtNetwork/QHostAddress>
 #include <QtNetwork/QNetworkProxy>
 
-
 static KTcpSocket::SslVersion kSslVersionFromQ(QSsl::SslProtocol protocol)
 {
     switch (protocol) {
@@ -45,6 +44,12 @@ static KTcpSocket::SslVersion kSslVersionFromQ(QSsl::SslProtocol protocol)
         return KTcpSocket::TlsV1;
     case QSsl::AnyProtocol:
         return KTcpSocket::AnySslVersion;
+#if QT_VERSION >= 0x040800
+    case QSsl::TlsV1SslV3:
+        return KTcpSocket::TlsV1SslV3;
+    case QSsl::SecureProtocols:
+        return KTcpSocket::SecureProtocols;
+#endif
     default:
         return KTcpSocket::UnknownSslVersion;
     }
@@ -69,6 +74,13 @@ static QSsl::SslProtocol qSslProtocolFromK(KTcpSocket::SslVersion sslVersion)
         return QSsl::SslV3;
     case KTcpSocket::TlsV1:
         return QSsl::TlsV1;
+#if QT_VERSION >= 0x040800
+    case KTcpSocket::TlsV1SslV3:
+        return QSsl::TlsV1SslV3;
+    case KTcpSocket::SecureProtocols:
+        return QSsl::SecureProtocols;
+#endif
+
     default:
         //QSslSocket doesn't really take arbitrary combinations. It's one or all.
         return QSsl::AnyProtocol;
@@ -358,7 +370,7 @@ public:
     void maybeLoadCertificates()
     {
         if (!certificatesLoaded) {
-            sock.setCaCertificates(KSslCertificateManager::self()->rootCertificates());
+            sock.setCaCertificates(KSslCertificateManager::self()->caCertificates());
             certificatesLoaded = true;
         }
     }
@@ -381,16 +393,19 @@ KTcpSocket::KTcpSocket(QObject *parent)
 
     connect(&d->sock, SIGNAL(aboutToClose()), this, SIGNAL(aboutToClose()));
     connect(&d->sock, SIGNAL(bytesWritten(qint64)), this, SIGNAL(bytesWritten(qint64)));
+    connect(&d->sock, SIGNAL(encryptedBytesWritten(qint64)), this, SIGNAL(encryptedBytesWritten(qint64)));
     connect(&d->sock, SIGNAL(readyRead()), this, SLOT(reemitReadyRead()));
     connect(&d->sock, SIGNAL(connected()), this, SIGNAL(connected()));
     connect(&d->sock, SIGNAL(encrypted()), this, SIGNAL(encrypted()));
     connect(&d->sock, SIGNAL(disconnected()), this, SIGNAL(disconnected()));
-    connect(&d->sock, SIGNAL(proxyAuthenticationRequired(const QNetworkProxy &, QAuthenticator *)),
-            this, SIGNAL(proxyAuthenticationRequired(const QNetworkProxy &, QAuthenticator *)));
+#ifndef QT_NO_NETWORKPROXY
+    connect(&d->sock, SIGNAL(proxyAuthenticationRequired(QNetworkProxy,QAuthenticator*)),
+            this, SIGNAL(proxyAuthenticationRequired(QNetworkProxy,QAuthenticator*)));
+#endif
     connect(&d->sock, SIGNAL(error(QAbstractSocket::SocketError)),
             this, SLOT(reemitSocketError(QAbstractSocket::SocketError)));
-    connect(&d->sock, SIGNAL(sslErrors(const QList<QSslError> &)),
-            this, SLOT(reemitSslErrors(const QList<QSslError> &)));
+    connect(&d->sock, SIGNAL(sslErrors(QList<QSslError>)),
+            this, SLOT(reemitSslErrors(QList<QSslError>)));
     connect(&d->sock, SIGNAL(hostFound()), this, SIGNAL(hostFound()));
     connect(&d->sock, SIGNAL(stateChanged(QAbstractSocket::SocketState)),
             this, SLOT(reemitStateChanged(QAbstractSocket::SocketState)));
@@ -579,11 +594,12 @@ quint16 KTcpSocket::peerPort() const
 }
 
 
+#ifndef QT_NO_NETWORKPROXY
 QNetworkProxy KTcpSocket::proxy() const
 {
     return d->sock.proxy();
 }
-
+#endif
 
 qint64 KTcpSocket::readBufferSize() const
 {
@@ -591,11 +607,12 @@ qint64 KTcpSocket::readBufferSize() const
 }
 
 
+#ifndef QT_NO_NETWORKPROXY
 void KTcpSocket::setProxy(const QNetworkProxy &proxy)
 {
     d->sock.setProxy(proxy);
 }
-
+#endif
 
 void KTcpSocket::setReadBufferSize(qint64 size)
 {
@@ -730,21 +747,42 @@ void KTcpSocket::setLocalCertificate(const QString &fileName, QSsl::EncodingForm
 }
 
 
-//TODO
-void KTcpSocket::setPrivateKey(const KSslKey &key)
+void KTcpSocket::setVerificationPeerName(const QString& hostName)
 {
-    Q_UNUSED(key)
+#if QT_VERSION >= 0x040800
+    d->sock.setPeerVerifyName(hostName);
+#else
+    Q_UNUSED(hostName);
+#endif
+}
+
+
+void KTcpSocket::setPrivateKey(const KSslKey &key)
+{
+    // We cannot map KSslKey::Algorithm:Dh to anything in QSsl::KeyAlgorithm.
+    if (key.algorithm() == KSslKey::Dh)
+        return;
+
+    QSslKey _key(key.toDer(),
+        (key.algorithm() == KSslKey::Rsa) ? QSsl::Rsa : QSsl::Dsa,
+        QSsl::Der,
+        (key.secrecy() == KSslKey::PrivateKey) ? QSsl::PrivateKey : QSsl::PublicKey);
+
+    d->sock.setPrivateKey(_key);
 }
 
 
-//TODO
 void KTcpSocket::setPrivateKey(const QString &fileName, KSslKey::Algorithm algorithm,
                                QSsl::EncodingFormat format, const QByteArray &passPhrase)
 {
-    Q_UNUSED(fileName)
-    Q_UNUSED(algorithm)
-    Q_UNUSED(format)
-    Q_UNUSED(passPhrase)
+    // We cannot map KSslKey::Algorithm:Dh to anything in QSsl::KeyAlgorithm.
+    if (algorithm == KSslKey::Dh)
+        return;
+
+    d->sock.setPrivateKey(fileName,
+        (algorithm == KSslKey::Rsa) ? QSsl::Rsa : QSsl::Dsa,
+        format,
+        passPhrase);
 }
 
 
@@ -766,7 +804,7 @@ QVariant KTcpSocket::socketOption(QAbstractSocket::SocketOption options) const
 
 void KTcpSocket::setSocketOption(QAbstractSocket::SocketOption options, const QVariant &value)
 {
-    return d->sock.setSocketOption(options, value);
+    d->sock.setSocketOption(options, value);
 }
 
 //slot
@@ -947,7 +985,7 @@ KSslCipher::KSslCipher(const QSslCipher &qsc)
     d->encryptionMethod = qsc.encryptionMethod();
     //Qt likes to append the number of bits (usedBits?) to the algorithm,
     //for example "AES(256)". We only want the pure algorithm name, though.
-    int parenIdx = d->encryptionMethod.indexOf('(');
+    int parenIdx = d->encryptionMethod.indexOf(QLatin1Char('('));
     if (parenIdx > 0)
         d->encryptionMethod.truncate(parenIdx);
     d->keyExchangeMethod = qsc.keyExchangeMethod();
@@ -1000,11 +1038,11 @@ QString KSslCipher::digestMethod() const
     //### This is not really backend neutral. It works for OpenSSL and
     //    for RFC compliant names, though.
     if (d->name.endsWith(QLatin1String("SHA")))
-        return "SHA-1";
+        return QString::fromLatin1("SHA-1");
     else if (d->name.endsWith(QLatin1String("MD5")))
-        return "MD5";
+        return QString::fromLatin1("MD5");
     else
-        return "";
+        return QString::fromLatin1(""); // ## probably QString() is enough
 }
 
 
@@ -1059,11 +1097,34 @@ KSslErrorUiData::KSslErrorUiData(const KTcpSocket *socket)
     d->bits = socket->sessionCipher().supportedBits();
 }
 
+KSslErrorUiData::KSslErrorUiData(const QSslSocket *socket)
+ : d(new Private())
+{
+    d->certificateChain = socket->peerCertificateChain();
+
+    // See KTcpSocket::sslErrors()
+    foreach (const QSslError &e, socket->sslErrors())
+        d->sslErrors.append(KSslError(e));
+
+    d->ip = socket->peerAddress().toString();
+    d->host = socket->peerName();
+    if (socket->isEncrypted()) {
+        d->sslProtocol = socket->sessionCipher().protocolString();
+    }
+    d->cipher = socket->sessionCipher().name();
+    d->usedBits = socket->sessionCipher().usedBits();
+    d->bits = socket->sessionCipher().supportedBits();
+}
+
 
 KSslErrorUiData::KSslErrorUiData(const KSslErrorUiData &other)
  : d(new Private(*other.d))
 {}
 
+KSslErrorUiData::~KSslErrorUiData()
+{
+    delete d;
+}
 
 KSslErrorUiData &KSslErrorUiData::operator=(const KSslErrorUiData &other)
 {
