@@ -24,13 +24,10 @@
 
 #include <kdebug.h>
 #include <kglobal.h>
-#include <kstandarddirs.h>
-#include <kcharsets.h>
 #include <klocale.h>
+#include <kstandarddirs.h>
 #include <kprotocolinfo.h>
-#include <kcomponentdata.h>
 #include <kcolorscheme.h>
-#include <kapplication.h>
 
 #include <kstyle.h>
 
@@ -48,6 +45,7 @@
 #include <QtDBus/QtDBus>
 #include <QtGui/QStyleFactory>
 #include <QDesktopServices>
+#include "qplatformdefs.h"
 
 // next two needed so we can set their palettes
 #include <QtGui/QToolTip>
@@ -78,9 +76,11 @@ static QRgb qt_colorref2qrgb(COLORREF col)
 //static QColor *_buttonBackground = 0;
 static KGlobalSettings::GraphicEffects _graphicEffects = KGlobalSettings::NoEffects;
 
-// KDE5: merge this with KGlobalSettings::Private
-// also think to make all methods static and not expose an object,
+// TODO: merge this with KGlobalSettings::Private
+//
+// F. Kossebau: KDE5: think to make all methods static and not expose an object,
 // making KGlobalSettings rather a namespace
+// D. Faure: how would people connect to signals, then?
 class KGlobalSettingsData
 {
   public:
@@ -148,10 +148,13 @@ class KGlobalSettings::Private
 {
     public:
         Private(KGlobalSettings *q)
-            : q(q), activated(false)
+            : q(q), activated(false), paletteCreated(false)
         {
+            kdeFullSession = !qgetenv("KDE_FULL_SESSION").isEmpty();
         }
 
+        QPalette createApplicationPalette(const KSharedConfigPtr &config);
+        QPalette createNewApplicationPalette(const KSharedConfigPtr &config);
         void _k_slotNotifyChange(int, int);
 
         void propagateQtSettings();
@@ -173,13 +176,13 @@ class KGlobalSettings::Private
          */
         void applyCursorTheme();
 
-        /**
-         * drop cached values for settings that aren't in any of the previous groups
-         */
-        static void rereadOtherSettings();
+        static void reloadStyleSettings();
 
         KGlobalSettings *q;
         bool activated;
+        bool paletteCreated;
+        bool kdeFullSession;
+        QPalette applicationPalette;
 };
 
 KGlobalSettings* KGlobalSettings::self()
@@ -191,8 +194,6 @@ KGlobalSettings* KGlobalSettings::self()
 KGlobalSettings::KGlobalSettings()
     : QObject(0), d(new Private(this))
 {
-    QDBusConnection::sessionBus().connect( QString(), "/KGlobalSettings", "org.kde.KGlobalSettings",
-                                           "notifyChange", this, SLOT(_k_slotNotifyChange(int,int)) );
 }
 
 KGlobalSettings::~KGlobalSettings()
@@ -201,13 +202,25 @@ KGlobalSettings::~KGlobalSettings()
 }
 
 void KGlobalSettings::activate()
+{
+    activate(ApplySettings | ListenForChanges);
+}
+
+void KGlobalSettings::activate(ActivateOptions options)
 {
     if (!d->activated) {
         d->activated = true;
 
-        d->kdisplaySetStyle(); // implies palette setup
-        d->kdisplaySetFont();
-        d->propagateQtSettings();
+        if (options & ListenForChanges) {
+            QDBusConnection::sessionBus().connect( QString(), "/KGlobalSettings", "org.kde.KGlobalSettings",
+                                                   "notifyChange", this, SLOT(_k_slotNotifyChange(int,int)) );
+        }
+
+        if (options & ApplySettings) {
+            d->kdisplaySetStyle(); // implies palette setup
+            d->kdisplaySetFont();
+            d->propagateQtSettings();
+        }
     }
 }
 
@@ -270,6 +283,7 @@ bool KGlobalSettings::showContextMenusOnPress ()
     return g.readEntry("ShowOnPress", true);
 }
 
+#ifndef KDE_NO_DEPRECATED
 int KGlobalSettings::contextMenuKey ()
 {
     KConfigGroup g(KGlobal::config(), "Shortcuts");
@@ -297,6 +311,7 @@ int KGlobalSettings::contextMenuKey ()
 
     return QKeySequence::fromString(s)[0];
 }
+#endif
 
 // NOTE: keep this in sync with kdebase/workspace/kcontrol/colors/colorscm.cpp
 QColor KGlobalSettings::inactiveTitleColor()
@@ -305,7 +320,7 @@ QColor KGlobalSettings::inactiveTitleColor()
     return qt_colorref2qrgb(GetSysColor(COLOR_INACTIVECAPTION));
 #else
     KConfigGroup g( KGlobal::config(), "WM" );
-    return g.readEntry( "inactiveBackground", QColor(224, 223, 222) );
+    return g.readEntry( "inactiveBackground", QColor(224,223,222) );
 #endif
 }
 
@@ -316,7 +331,7 @@ QColor KGlobalSettings::inactiveTextColor()
     return qt_colorref2qrgb(GetSysColor(COLOR_INACTIVECAPTIONTEXT));
 #else
     KConfigGroup g( KGlobal::config(), "WM" );
-    return g.readEntry( "inactiveForeground", QColor(20, 19, 18) );
+    return g.readEntry( "inactiveForeground", QColor(75,71,67) );
 #endif
 }
 
@@ -327,7 +342,7 @@ QColor KGlobalSettings::activeTitleColor()
     return qt_colorref2qrgb(GetSysColor(COLOR_ACTIVECAPTION));
 #else
     KConfigGroup g( KGlobal::config(), "WM" );
-    return g.readEntry( "activeBackground", QColor(96, 148, 207));
+    return g.readEntry( "activeBackground", QColor(48,174,232));
 #endif
 }
 
@@ -338,21 +353,21 @@ QColor KGlobalSettings::activeTextColor()
     return qt_colorref2qrgb(GetSysColor(COLOR_CAPTIONTEXT));
 #else
     KConfigGroup g( KGlobal::config(), "WM" );
-    return g.readEntry( "activeForeground", QColor(255, 255, 255) );
+    return g.readEntry( "activeForeground", QColor(255,255,255) );
 #endif
 }
 
 int KGlobalSettings::contrast()
 {
     KConfigGroup g( KGlobal::config(), "KDE" );
-    return g.readEntry( "contrast", 5 );
+    return g.readEntry( "contrast", 7 );
 }
 
 qreal KGlobalSettings::contrastF(const KSharedConfigPtr &config)
 {
     if (config) {
         KConfigGroup g( config, "KDE" );
-        return 0.1 * g.readEntry( "contrast", 5 );
+        return 0.1 * g.readEntry( "contrast", 7 );
     }
     return 0.1 * (qreal)contrast();
 }
@@ -393,6 +408,11 @@ static const KFontData DefaultFontData[KGlobalSettingsData::FontTypesCount] =
     { GeneralId, "fixed",       "Monaco",       10, -1, QFont::TypeWriter },
     { GeneralId, "toolBarFont", DefaultMacFont, 11, -1, QFont::SansSerif },
     { GeneralId, "menuFont",    DefaultMacFont, 13, -1, QFont::SansSerif },
+#elif defined(Q_WS_MAEMO_5) || defined(MEEGO_EDITION_HARMATTAN)
+    { GeneralId, "font",        DefaultFont, 16, -1, QFont::SansSerif },
+    { GeneralId, "fixed",       "Monospace", 16, -1, QFont::TypeWriter },
+    { GeneralId, "toolBarFont", DefaultFont, 16, -1, QFont::SansSerif },
+    { GeneralId, "menuFont",    DefaultFont, 16, -1, QFont::SansSerif },
 #else
     { GeneralId, "font",        DefaultFont, 9, -1, QFont::SansSerif },
     { GeneralId, "fixed",       "Monospace", 9, -1, QFont::TypeWriter },
@@ -578,9 +598,14 @@ KGlobalSettings::KMouseSettings& KGlobalSettingsData::mouseSettings()
     }
 #ifdef Q_WS_WIN
     //not cached
+#ifndef _WIN32_WCE
     mMouseSettings->handed = (GetSystemMetrics(SM_SWAPBUTTON) ?
         KGlobalSettings::KMouseSettings::LeftHanded :
         KGlobalSettings::KMouseSettings::RightHanded);
+#else
+// There is no mice under wince
+    mMouseSettings->handed =KGlobalSettings::KMouseSettings::RightHanded;
+#endif
 #endif
     return *mMouseSettings;
 }
@@ -634,6 +659,9 @@ QString KGlobalSettings::downloadPath()
         KConfig xdgUserConf( xdgUserDirs, KConfig::SimpleConfig );
         KConfigGroup g( &xdgUserConf, "" );
         downloadPath  = g.readPathEntry( "XDG_DOWNLOAD_DIR", downloadPath ).remove(  '"' );
+        if ( downloadPath.isEmpty() ) {
+            downloadPath = QDir::homePath();
+        }
     }
 #endif
     downloadPath = QDir::cleanPath( downloadPath );
@@ -758,7 +786,7 @@ KGlobalSettings::GraphicEffects KGlobalSettings::graphicEffectsLevel()
 
     if (!_graphicEffectsInitialized) {
         _graphicEffectsInitialized = true;
-        Private::rereadOtherSettings();
+        Private::reloadStyleSettings();
     }
 
     return _graphicEffects;
@@ -827,6 +855,7 @@ void KGlobalSettings::Private::_k_slotNotifyChange(int changeType, int arg)
     case PaletteChanged:
         if (activated) {
             KGlobal::config()->reparseConfiguration();
+            paletteCreated = false;
             kdisplaySetPalette();
         }
         break;
@@ -841,16 +870,25 @@ void KGlobalSettings::Private::_k_slotNotifyChange(int changeType, int arg)
 
     case SettingsChanged: {
         KGlobal::config()->reparseConfiguration();
-        rereadOtherSettings();
         SettingsCategory category = static_cast<SettingsCategory>(arg);
-        if (category == SETTINGS_MOUSE) {
-            KGlobalSettingsData::self()->dropMouseSettingsCache();
-        }
         if (category == SETTINGS_QT) {
             if (activated) {
                 propagateQtSettings();
             }
         } else {
+            switch (category) {
+                case SETTINGS_STYLE:
+                    reloadStyleSettings();
+                    break;
+                case SETTINGS_MOUSE:
+                    KGlobalSettingsData::self()->dropMouseSettingsCache();
+                    break;
+                case SETTINGS_LOCALE:
+                    KGlobal::locale()->reparseConfiguration();
+                    break;
+                default:
+                    break;
+            }
             emit q->settingsChanged(category);
         }
         break;
@@ -876,7 +914,7 @@ void KGlobalSettings::Private::_k_slotNotifyChange(int changeType, int arg)
         break;
 
     default:
-        kWarning(101) << "Unknown type of change in KGlobalSettings::slotNotifyChange: " << changeType;
+        kWarning(240) << "Unknown type of change in KGlobalSettings::slotNotifyChange: " << changeType;
     }
 }
 
@@ -885,6 +923,18 @@ QString kde_overrideStyle;
 
 void KGlobalSettings::Private::applyGUIStyle()
 {
+  //Platform plugin only loaded on X11 systems
+#ifdef Q_WS_X11
+    if (!kde_overrideStyle.isEmpty()) {
+        const QLatin1String currentStyleName(qApp->style()->metaObject()->className());
+        if (0 != kde_overrideStyle.compare(currentStyleName, Qt::CaseInsensitive) &&
+            0 != (QString(kde_overrideStyle + QLatin1String("Style"))).compare(currentStyleName, Qt::CaseInsensitive)) {
+            qApp->setStyle(kde_overrideStyle);
+        }
+    } else {
+        emit q->kdisplayStyleChanged();
+    }
+#else
     const QLatin1String currentStyleName(qApp->style()->metaObject()->className());
 
     if (kde_overrideStyle.isEmpty()) {
@@ -917,9 +967,30 @@ void KGlobalSettings::Private::applyGUIStyle()
         qApp->setStyle(kde_overrideStyle);
     }
     emit q->kdisplayStyleChanged();
+#endif //Q_WS_X11
 }
 
 QPalette KGlobalSettings::createApplicationPalette(const KSharedConfigPtr &config)
+{
+    return self()->d->createApplicationPalette(config);
+}
+
+QPalette KGlobalSettings::createNewApplicationPalette(const KSharedConfigPtr &config)
+{
+    return self()->d->createNewApplicationPalette(config);
+}
+
+QPalette KGlobalSettings::Private::createApplicationPalette(const KSharedConfigPtr &config)
+{
+    // This method is typically called once by KQGuiPlatformPlugin::palette and once again
+    // by kdisplaySetPalette(), so we cache the palette to save time.
+    if (config == KGlobal::config() && paletteCreated) {
+        return applicationPalette;
+    }
+    return createNewApplicationPalette(config);
+}
+
+QPalette KGlobalSettings::Private::createNewApplicationPalette(const KSharedConfigPtr &config)
 {
     QPalette palette;
 
@@ -958,26 +1029,37 @@ QPalette KGlobalSettings::createApplicationPalette(const KSharedConfigPtr &confi
         palette.setBrush( state, QPalette::LinkVisited, schemeView.foreground( KColorScheme::VisitedText ) );
     }
 
+    if (config == KGlobal::config()) {
+        paletteCreated = true;
+        applicationPalette = palette;
+    }
+
     return palette;
 }
 
 void KGlobalSettings::Private::kdisplaySetPalette()
 {
-    // Added by Sam/Harald (TT) for Mac OS X initially, but why?
-    KConfigGroup cg( KGlobal::config(), "General" );
-    if (cg.readEntry("nopaletteChange", false))
+#if !defined(Q_WS_MAEMO_5) && !defined(Q_OS_WINCE) && !defined(MEEGO_EDITION_HARMATTAN)
+    if (!kdeFullSession) {
         return;
+    }
 
     if (qApp->type() == QApplication::GuiClient) {
         QApplication::setPalette( q->createApplicationPalette() );
     }
     emit q->kdisplayPaletteChanged();
     emit q->appearanceChanged();
+#endif    
 }
 
 
 void KGlobalSettings::Private::kdisplaySetFont()
 {
+#if !defined(Q_WS_MAEMO_5) && !defined(Q_OS_WINCE) && !defined(MEEGO_EDITION_HARMATTAN)
+    if (!kdeFullSession) {
+        return;
+    }
+
     if (qApp->type() == QApplication::GuiClient) {
         KGlobalSettingsData* data = KGlobalSettingsData::self();
 
@@ -990,6 +1072,7 @@ void KGlobalSettings::Private::kdisplaySetFont()
     }
     emit q->kdisplayFontChanged();
     emit q->appearanceChanged();
+#endif
 }
 
 
@@ -1004,7 +1087,7 @@ void KGlobalSettings::Private::kdisplaySetStyle()
 }
 
 
-void KGlobalSettings::Private::rereadOtherSettings()
+void KGlobalSettings::Private::reloadStyleSettings()
 {
     KConfigGroup g( KGlobal::config(), "KDE-Global GUI Settings" );
 
@@ -1068,6 +1151,8 @@ void KGlobalSettings::Private::propagateQtSettings()
     QApplication::setStartDragDistance(num);
     num = cg.readEntry("WheelScrollLines", QApplication::wheelScrollLines());
     QApplication::setWheelScrollLines(num);
+    bool showIcons = cg.readEntry("ShowIconsInMenuItems", !QApplication::testAttribute(Qt::AA_DontShowIconsInMenus));
+    QApplication::setAttribute(Qt::AA_DontShowIconsInMenus, !showIcons);
 
     // KDE5: this seems fairly pointless
     emit q->settingsChanged(SETTINGS_QT);
