@@ -200,6 +200,28 @@ void KRichTextEdit::alignJustify()
     d->activateRichText();
 }
 
+void KRichTextEdit::makeRightToLeft()
+{
+    QTextBlockFormat format;
+    format.setLayoutDirection(Qt::RightToLeft);
+    QTextCursor cursor = textCursor();
+    cursor.mergeBlockFormat(format);
+    setTextCursor(cursor);
+    setFocus();
+    d->activateRichText();
+}
+
+void KRichTextEdit::makeLeftToRight()
+{
+    QTextBlockFormat format;
+    format.setLayoutDirection(Qt::LeftToRight);
+    QTextCursor cursor = textCursor();
+    cursor.mergeBlockFormat(format);
+    setTextCursor(cursor);
+    setFocus();
+    d->activateRichText();
+}
+
 void KRichTextEdit::setTextBold(bool bold)
 {
     QTextCharFormat fmt;
@@ -500,9 +522,57 @@ bool KRichTextEdit::canDedentList() const
 
 QString KRichTextEdit::toCleanHtml() const
 {
-    // Does nothing anymore, as Qt seems to be fixed now.
-    // See the unit test which confirms this.
-    return toHtml();
+  QString result = toHtml();
+
+  static const QString EMPTYLINEHTML = QLatin1String(
+  "<p style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; "
+  "margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px; \">&nbsp;</p>" );
+
+  // Qt inserts various style properties based on the current mode of the editor (underline, 
+  // bold, etc), but only empty paragraphs *also* have qt-paragraph-type set to 'empty'.
+  static const QString EMPTYLINEREGEX = QLatin1String(
+    "<p style=\"-qt-paragraph-type:empty;(.*)</p>" );
+  
+  static const QString OLLISTPATTERNQT = QLatin1String(
+  "<ol style=\"margin-top: 0px; margin-bottom: 0px; margin-left: 0px;" );
+
+  static const QString ULLISTPATTERNQT = QLatin1String(
+  "<ul style=\"margin-top: 0px; margin-bottom: 0px; margin-left: 0px;" );
+
+  static const QString ORDEREDLISTHTML = QLatin1String(
+  "<ol style=\"margin-top: 0px; margin-bottom: 0px;" );
+
+  static const QString UNORDEREDLISTHTML = QLatin1String(
+  "<ul style=\"margin-top: 0px; margin-bottom: 0px;" );
+
+  // fix 1 - empty lines should show as empty lines - MS Outlook treats margin-top:0px; as
+  // a non-existing line.
+  // Although we can simply remove the margin-top style property, we still get unwanted results
+  // if you have three or more empty lines. It's best to replace empty <p> elements with <p>&nbsp;</p>.
+
+  QRegExp emptyLineFinder( EMPTYLINEREGEX );
+  emptyLineFinder.setMinimal( true );
+  
+  // find the first occurance
+  int offset = emptyLineFinder.indexIn( result, 0 );
+  while (offset != -1) {
+    // replace all the matching text with the new line text
+    result.replace( offset, emptyLineFinder.matchedLength(), EMPTYLINEHTML );
+    // advance the search offset to just beyond the last replace
+    offset += EMPTYLINEHTML.length();
+    // find the next occurance
+    offset = emptyLineFinder.indexIn( result, offset );
+  }
+  
+  // fix 2a - ordered lists - MS Outlook treats margin-left:0px; as
+  // a non-existing number; e.g: "1. First item" turns into "First Item"
+  result.replace(OLLISTPATTERNQT, ORDEREDLISTHTML);
+
+  // fix 2b - unordered lists - MS Outlook treats margin-left:0px; as
+  // a non-existing bullet; e.g: "* First bullet" turns into "First Bullet"
+  result.replace(ULLISTPATTERNQT, UNORDEREDLISTHTML);
+
+  return result;
 }
 
 #include "krichtextedit.moc"
