@@ -28,8 +28,14 @@
 #include <klocale.h>
 #include <kmessagebox.h>
 #include <khtml_filter_p.h>
+#include <kstandarddirs.h>
+#include <kjob.h>
+#include <kio/job.h>
 
+#include <QFile>
+#include <QFileInfo>
 #include <QtGui/QFontDatabase>
+#include <QByteArray>
 
 /**
  * @internal
@@ -64,12 +70,12 @@ struct KPerDomainSettings {
 QString *KHTMLSettings::avFamilies = 0;
 typedef QMap<QString,KPerDomainSettings> PolicyMap;
 
-class KHTMLSettingsPrivate
+// The "struct" that contains all the data. Must be copiable (no pointers).
+class KHTMLSettingsData
 {
 public:
     bool m_bChangeCursor : 1;
     bool m_bOpenMiddleClick : 1;
-    bool m_bBackRightClick : 1;
     bool m_underlineLink : 1;
     bool m_hoverLink : 1;
     bool m_bEnableJavaScriptDebug : 1;
@@ -115,6 +121,81 @@ public:
     QList< QPair< QString, QChar > > m_fallbackAccessKeysAssignments;
 };
 
+class KHTMLSettingsPrivate : public QObject, public KHTMLSettingsData
+{
+    Q_OBJECT
+public:
+
+    void adblockFilterLoadList(const QString& filename)
+    {
+        kDebug(6000) << "Loading filter list from" << filename;
+        /** load list file and process each line */
+        QFile file(filename);
+        if (file.open(QIODevice::ReadOnly)) {
+            QTextStream ts(&file);
+            QString line = ts.readLine();
+#ifndef NDEBUG /// only count when compiled for debugging
+            int whiteCounter = 0, blackCounter = 0;
+#endif // NDEBUG
+            while (!line.isEmpty()) {
+                /** white list lines start with "@@" */
+                if (line.startsWith(QLatin1String("@@")))
+                {
+#ifndef NDEBUG
+                    ++whiteCounter;
+#endif // NDEBUG
+                    adWhiteList.addFilter(line);
+                }
+                else
+                {
+#ifndef NDEBUG
+                    ++blackCounter;
+#endif // NDEBUG
+                    adBlackList.addFilter(line);
+                }
+
+                line = ts.readLine();
+            }
+            file.close();
+
+#ifndef NDEBUG
+            kDebug(6000) << "Filter list loaded" << whiteCounter << "white list entries and" << blackCounter << "black list entries";
+#endif // NDEBUG
+        }
+    }
+
+public slots:
+    void adblockFilterResult(KJob *job)
+    {
+      KIO::StoredTransferJob *tJob = qobject_cast<KIO::StoredTransferJob*>(job);
+      Q_ASSERT(tJob);
+
+      if ( tJob->error() ) {
+          kDebug(6000) << "Failed to download" << tJob->url() << "with message:" << tJob->errorText();
+      }
+      else if ( tJob->isErrorPage() ) { // 4XX error code
+          kDebug(6000) << "Failed to fetch filter list" << tJob->url();
+      }
+      else {
+          const QByteArray byteArray = tJob->data();
+          const QString localFileName = tJob->property( "khtmlsettings_adBlock_filename" ).toString();
+
+          QFile file(localFileName);
+          if ( file.open(QFile::WriteOnly) ) {
+              bool success = file.write(byteArray) == byteArray.size();
+              file.close();
+              if ( success )
+                  adblockFilterLoadList(localFileName);
+              else
+                  kDebug(6000) << "Could not write" << byteArray.size() << "to file" << localFileName;
+          }
+          else
+              kDebug(6000) << "Cannot open file" << localFileName << "for filter list";
+      }
+
+    }
+};
+
 
 /** Returns a writeable per-domains settings instance for the given domain
   * or a deep copy of the global settings if not existent.
@@ -123,7 +204,7 @@ static KPerDomainSettings &setup_per_domain_policy(
 				KHTMLSettingsPrivate* const d,
 				const QString &domain) {
   if (domain.isEmpty()) {
-    kWarning() << "setup_per_domain_policy: domain is empty";
+    kWarning(6000) << "setup_per_domain_policy: domain is empty";
   }
   const QString ldomain = domain.toLower();
   PolicyMap::iterator it = d->domainPolicy.find(ldomain);
@@ -269,7 +350,8 @@ KHTMLSettings::KHTMLSettings()
 KHTMLSettings::KHTMLSettings(const KHTMLSettings &other)
 	:d(new KHTMLSettingsPrivate())
 {
-  *d = *other.d;
+  KHTMLSettingsData* data = d;
+  *data = *other.d;
 }
 
 KHTMLSettings::~KHTMLSettings()
@@ -311,9 +393,6 @@ void KHTMLSettings::init( KConfig * config, bool reset )
   {
     if ( reset || cg.hasKey( "OpenMiddleClick" ) )
         d->m_bOpenMiddleClick = cg.readEntry( "OpenMiddleClick", true );
-
-    if ( reset || cg.hasKey( "BackRightClick" ) )
-        d->m_bBackRightClick = cg.readEntry( "BackRightClick", false );
   }
 
   KConfigGroup cgAccess(config,"Access Keys" );
@@ -331,21 +410,63 @@ void KHTMLSettings::init( KConfig * config, bool reset )
       d->adBlackList.clear();
       d->adWhiteList.clear();
 
-      QMap<QString,QString> entryMap = cgFilter.entryMap();
-      QMap<QString,QString>::ConstIterator it;
-      for( it = entryMap.constBegin(); it != entryMap.constEnd(); ++it )
-      {
-          QString name = it.key();
-          QString url = it.value();
+      if (d->m_adFilterEnabled) {
 
-          if (name.startsWith("Filter"))
+          /** read maximum age for filter list files, minimum is one day */
+          int htmlFilterListMaxAgeDays = cgFilter.readEntry(QString("HTMLFilterListMaxAgeDays")).toInt();
+          if (htmlFilterListMaxAgeDays < 1)
+              htmlFilterListMaxAgeDays = 1;
+
+          QMap<QString,QString> entryMap = cgFilter.entryMap();
+          QMap<QString,QString>::ConstIterator it;
+          for( it = entryMap.constBegin(); it != entryMap.constEnd(); ++it )
           {
-              if (url.startsWith(QLatin1String("@@")))
-                  d->adWhiteList.addFilter(url);
-              else
-                  d->adBlackList.addFilter(url);
+              int id = -1;
+              QString name = it.key();
+              QString url = it.value();
+
+              if (name.startsWith("Filter"))
+              {
+                  if (url.startsWith(QLatin1String("@@")))
+                      d->adWhiteList.addFilter(url);
+                  else
+                      d->adBlackList.addFilter(url);
+              } else if (name.startsWith("HTMLFilterListName-") && (id = name.mid(19).toInt()) > 0)
+              {
+                  /** check if entry is enabled */
+                  bool filterEnabled = cgFilter.readEntry(QString("HTMLFilterListEnabled-").append(QString::number(id))) != QLatin1String("false");
+
+                  /** get url for HTMLFilterList */
+                  KUrl url(cgFilter.readEntry(QString("HTMLFilterListURL-").append(QString::number(id))));
+
+                  if (filterEnabled && url.isValid()) {
+                      /** determine where to cache HTMLFilterList file */
+                      QString localFile = cgFilter.readEntry(QString("HTMLFilterListLocalFilename-").append(QString::number(id)));
+                      localFile = KStandardDirs::locateLocal("data", "khtml/" + localFile);
+
+                      /** determine existence and age of cache file */
+                      QFileInfo fileInfo(localFile);
+
+                      /** load cached file if it exists, irrespective of age */
+                      if (fileInfo.exists())
+                          d->adblockFilterLoadList( localFile );
+
+                      /** if no cache list file exists or if it is too old ... */
+                      if (!fileInfo.exists() || fileInfo.lastModified().daysTo(QDateTime::currentDateTime()) > htmlFilterListMaxAgeDays)
+                      {
+                          /** ... in this case, refetch list asynchronously */
+                          kDebug(6000) << "Asynchronously fetching filter list from" << url << "to" << localFile;
+
+                          KIO::StoredTransferJob *job = KIO::storedGet( url, KIO::Reload, KIO::HideProgressInfo );
+                          QObject::connect( job, SIGNAL(result(KJob*)), d, SLOT(adblockFilterResult(KJob*)) );
+                          /** for later reference, store name of cache file */
+                          job->setProperty("khtmlsettings_adBlock_filename", localFile);
+                      }
+                  }
+              }
           }
       }
+
   }
 
   KConfigGroup cgHtml( config, "HTML Settings" );
@@ -712,7 +833,7 @@ bool KHTMLSettings::isOpenMiddleClickEnabled()
 
 bool KHTMLSettings::isBackRightClickEnabled()
 {
-  return d->m_bBackRightClick;
+  return false; // ## the feature moved to konqueror
 }
 
 bool KHTMLSettings::accessKeysEnabled() const
@@ -743,12 +864,33 @@ bool KHTMLSettings::isAdFiltered( const QString &url ) const
     return false;
 }
 
+QString KHTMLSettings::adFilteredBy( const QString &url, bool *isWhiteListed ) const
+{
+    QString m = d->adWhiteList.urlMatchedBy(url);
+    if (!m.isEmpty())
+    {
+        if (isWhiteListed != 0)
+            *isWhiteListed = true;
+        return (m);
+    }
+
+    m = d->adBlackList.urlMatchedBy(url);
+    if (!m.isEmpty())
+    {
+        if (isWhiteListed != 0)
+            *isWhiteListed = false;
+        return (m);
+    }
+
+    return (QString());
+}
+
 void KHTMLSettings::addAdFilter( const QString &url )
 {
     KConfigGroup config = KSharedConfig::openConfig( "khtmlrc", KConfig::NoGlobals )->group( "Filter Settings" );
 
     QRegExp rx;
-    
+
     // Try compiling to avoid invalid stuff. Only support the basic syntax here...
     // ### refactor somewhat
     if (url.length()>2 && url[0]=='/' && url[url.length()-1] == '/')
@@ -1072,3 +1214,5 @@ bool KHTMLSettings::jsPopupBlockerPassivePopup() const
 {
     return d->m_jsPopupBlockerPassivePopup;
 }
+
+#include "khtml_settings.moc"
