@@ -91,44 +91,38 @@ using namespace DOM;
         {
             noBorder = false;
             left = right = top = bottom = 0;
-            m_proxy = qobject_cast<KdeUiProxyStyle*>(parent->style());
             setParent(parent);
         }
 
-        QStyle* proxy() const { return m_proxy ? m_proxy : style(); }
-
-        QRect subElementRect(
-                SubElement element, const QStyleOption *option, const QWidget *widget
-            ) const
+        QRect subElementRect(SubElement element, const QStyleOption *option, const QWidget *widget) const
         {
-            QRect r = proxy()->subElementRect(element, option, widget);
+            QRect r = style()->subElementRect(element, option, widget);
             switch (element) {
               case QStyle::SE_PushButtonContents:
               case QStyle::SE_LineEditContents:
-              case QStyle::SE_FrameContents:
+              case QStyle::SE_ShapedFrameContents:
                 r.adjust(left, top, -right, -bottom);
               default:
                 break;
             }
             return r;
         }
+
         void drawControl(ControlElement element, const QStyleOption *option, QPainter *painter, const QWidget *widget) const
         {
             if ( noBorder && element == QStyle::CE_PushButton ) {
                const QStyleOptionButton *o = qstyleoption_cast<const QStyleOptionButton *>(option);
                if (o) {
                    QStyleOptionButton opt = *o;
-                   opt.rect = proxy()->subElementRect(SE_PushButtonFocusRect, &opt, widget);
-                   KdeUiProxyStyle::drawControl(CE_PushButtonLabel, &opt, painter, widget);
+                   opt.rect = style()->subElementRect(SE_PushButtonFocusRect, &opt, widget);
+                   style()->drawControl(CE_PushButtonLabel, &opt, painter, widget);
                }
                return;
             }
-            KdeUiProxyStyle::drawControl(element,option,painter,widget);
+            style()->drawControl(element,option,painter,widget);
         }
 
-
-        QRect subControlRect(ComplexControl cc, const QStyleOptionComplex* opt,
-                                SubControl sc, const QWidget* widget) const
+        QRect subControlRect(ComplexControl cc, const QStyleOptionComplex* opt, SubControl sc, const QWidget* widget) const
         {
             // Make sure we give combo popup's enough room to display contents;
             // Qt doesn't do this by default
@@ -149,23 +143,22 @@ using namespace DOM;
                     }
 
                     // Now let sizeFromContent add in extra stuff.
-                    maxW = proxy()->sizeFromContents(QStyle::CT_ComboBox, opt, QSize(maxW, 1), widget).width();
+                    maxW = style()->sizeFromContents(QStyle::CT_ComboBox, opt, QSize(maxW, 1), widget).width();
 
                     // How much more room do we need for the text?
                     int extraW = maxW > cbOpt->rect.width() ? maxW - cbOpt->rect.width() : 0;
 
-                    QRect r = proxy()->subControlRect(cc, opt, sc, widget);
+                    QRect r = style()->subControlRect(cc, opt, sc, widget);
                     r.setWidth(r.width() + extraW);
                     return r;
                 }
             }
 
-            return proxy()->subControlRect(cc, opt, sc, widget);
+            return style()->subControlRect(cc, opt, sc, widget);
         }
 
         int left, right, top, bottom;
         bool noBorder;
-        KdeUiProxyStyle* m_proxy;
     };
 
 // ---------------------------------------------------------------------
