@@ -861,7 +861,7 @@ void DocumentImpl::setTitle(const DOMString& _title)
             titleStr[i] = ' ';
     titleStr = titleStr.simplified();
     if ( view() && !view()->part()->parentPart() ) {
-	if (titleStr.isNull() || titleStr.isEmpty()) {
+	if (titleStr.isEmpty()) {
 	    // empty title... set window caption as the URL
 	    KUrl url = m_url;
 	    url.setRef(QString());
@@ -2824,6 +2824,7 @@ void DocumentImpl::dispatchImageLoadEventsNow()
 void DocumentImpl::timerEvent(QTimerEvent *e)
 {
     assert(e->timerId() == m_imageLoadEventTimer);
+    Q_UNUSED(e);
     dispatchImageLoadEventsNow();
 }
 
@@ -2840,38 +2841,51 @@ HTMLPartContainerElementImpl *DocumentImpl::ownerElement() const
     ChildFrame *childFrame = childPart->d->m_frame;
     if (!childFrame)
         return 0;
-    return childFrame->m_partContainerElement;
+    return childFrame->m_partContainerElement.data();
 }
 
+khtml::SecurityOrigin* DocumentImpl::origin() const
+{
+    if (!m_origin)
+        m_origin = SecurityOrigin::create(URL());
+    return m_origin.get();
+}
+
+void DocumentImpl::setOrigin(khtml::SecurityOrigin* newOrigin)
+{
+    assert(origin()->isEmpty());
+    m_origin = newOrigin;
+}
+    
 DOMString DocumentImpl::domain() const
 {
-    if ( m_domain.isEmpty() ) // not set yet (we set it on demand to save time and space)
-        m_domain = URL().host(); // Initially set to the host
-    return m_domain;
+    return origin()->domain();
 }
 
 void DocumentImpl::setDomain(const DOMString &newDomain)
 {
-    if ( m_domain.isEmpty() ) // not set yet (we set it on demand to save time and space)
-        m_domain = URL().host().toLower(); // Initially set to the host
-
-    if ( m_domain.isEmpty() /*&& part() && part()->openedByJS()*/ )
-        m_domain = newDomain.lower();
+    // ### this test really should move to SecurityOrigin..
+    DOMString oldDomain = origin()->domain();
 
     // Both NS and IE specify that changing the domain is only allowed when
     // the new domain is a suffix of the old domain.
-    int oldLength = m_domain.length();
+    int oldLength = oldDomain.length();
     int newLength = newDomain.length();
-    if ( newLength < oldLength ) // e.g. newDomain=kde.org (7) and m_domain=www.kde.org (11)
-    {
-        DOMString test = m_domain.copy();
+    if ( newLength < oldLength ) { // e.g. newDomain=kde.org (7) and m_domain=www.kde.org (11)
+        DOMString test = oldDomain.copy();
         DOMString reference = newDomain.lower();
         if ( test[oldLength - newLength - 1] == '.' ) // Check that it's a subdomain, not e.g. "de.org"
         {
             test.remove( 0, oldLength - newLength ); // now test is "kde.org" from m_domain
             if ( test == reference )                 // and we check that it's the same thing as newDomain
-                m_domain = reference;
+                m_origin->setDomainFromDOM( reference.string() );
         }
+    } else if ( oldLength == newLength ) {
+        // It's OK and not a no-op to set the domain to the present one:
+        // we want to set the 'set from DOM' bit in that case
+        DOMString reference = newDomain.lower();
+        if ( oldDomain.lower() == reference )
+            m_origin->setDomainFromDOM( reference.string() );
     }
 }
 
