@@ -28,6 +28,7 @@
 #include "file.h"
 
 #include <config.h>
+#include <config-kioslave-file.h>
 
 #include <QtCore/QFile>
 
@@ -48,6 +49,15 @@
 #include <limits.h>  // PATH_MAX
 #endif
 
+//sendfile has different semantics in different platforms
+#if defined HAVE_SENDFILE && defined Q_OS_LINUX
+#define USE_SENDFILE 1
+#endif
+
+#ifdef USE_SENDFILE
+#include <sys/sendfile.h>
+#endif
+
 namespace KDEPrivate
 {
 
@@ -145,7 +155,7 @@ void FileProtocol::copy( const KUrl &srcUrl, const KUrl &destUrl,
 	return;
     }
 
-#ifdef HAVE_FADVISE
+#if HAVE_FADVISE
     posix_fadvise(src_fd,0,0,POSIX_FADV_SEQUENTIAL);
 #endif
     // WABA: Make sure that we keep writing permissions ourselves,
@@ -168,7 +178,7 @@ void FileProtocol::copy( const KUrl &srcUrl, const KUrl &destUrl,
         return;
     }
 
-#ifdef HAVE_FADVISE
+#if HAVE_FADVISE
     posix_fadvise(dest_fd,0,0,POSIX_FADV_SEQUENTIAL);
 #endif
 
