@@ -70,6 +70,18 @@ void StatementNode::generateDebugInfo(CompileState* comp)
     CodeGen::emitOp(comp, Op_AtStatement, 0, &me);
 }
 
+static inline bool exitContextNeeded(CompileState* comp) {
+    return comp->compileType() == Debug &&
+           comp->codeType()    == FunctionCode;
+}
+
+static void generateExitContextIfNeeded(CompileState* comp) {
+    if (exitContextNeeded(comp)) {
+        OpValue ourNode = OpValue::immNode(comp->functionBody());
+        CodeGen::emitOp(comp, Op_ExitDebugContext, 0, &ourNode);        
+    }
+}
+
 // ------------------------------ Basic literals -----------------------------------------
 
 OpValue NullNode::generateEvalCode(CompileState*)
@@ -1255,18 +1267,19 @@ void ForInNode::generateExecCode(CompileState* comp)
 
     OpValue val = expr->generateEvalCode(comp);
     OpValue obj; // version of val after toObject, returned by BeginForIn.
+    
+    OpValue stateVal, stateReg;
+    comp->requestTemporary(OpType_value, &stateVal, &stateReg);
 
     // Fetch the property name array..
-    CodeGen::emitOp(comp, Op_BeginForIn, &obj, &val);
+    CodeGen::emitOp(comp, Op_BeginForIn, &obj, &val, &stateReg);
 
-    // ... as the array is store on an iterator stack, this introduces a cleanup entry.
-    comp->pushNest(CompileState::OtherCleanup, this);
-    
-    comp->enterLoop(this); // must do this here, since continue shouldn't pop our iterator!
+    comp->enterLoop(this); 
 
     // We put the test first here, since the test and the fetch are combined.
     OpValue sym;
-    Addr fetchNext = CodeGen::emitOp(comp, Op_NextForInEntry, &sym, &obj, OpValue::dummyAddr());
+    Addr fetchNext = CodeGen::emitOp(comp, Op_NextForInEntry, &sym, &obj,
+                                     &stateVal, OpValue::dummyAddr());
 
     // Write to the variable
     assert (lexpr->isLocation());
@@ -1286,16 +1299,10 @@ void ForInNode::generateExecCode(CompileState* comp)
     OpValue backVal = OpValue::immAddr(fetchNext);
     CodeGen::emitOp(comp, Op_Jump, 0, &backVal);
 
-    // The end address is here (#2 since return val..)
-    CodeGen::patchJumpToNext(comp, fetchNext, 2);
+    // The end address is here (3 argument + return val)
+    CodeGen::patchJumpToNext(comp, fetchNext, 3);
 
-    // The looping action ends here.. We need to do it before the EndForIn instruction so we always cleanup
-    // right on breaks.
     comp->exitLoop(this);
-    comp->popNest(); // Remove the cleanup entry.. Note that the breaks go to before here..    
-
-    // Cleanup
-    CodeGen::emitOp(comp, Op_EndForIn);
 }
 
 // Helper for continue/break -- emits stack cleanup call if needed,
@@ -1405,6 +1412,9 @@ void ReturnNode::generateExecCode(CompileState* comp)
     else
         arg = value->generateEvalCode(comp);
 
+    if (!comp->inTryFinally())
+        generateExitContextIfNeeded(comp);
+
     CodeGen::emitOp(comp, comp->inTryFinally() ? Op_ReturnInTryFinally : Op_Return, 0, &arg);
 }
 
@@ -1525,7 +1535,13 @@ void TryNode::generateExecCode(CompileState* comp)
         finallyBlock->generateExecCode(comp);
 
         OpValue otherTryFinally = OpValue::immBool(comp->inTryFinally());
-        CodeGen::emitOp(comp, Op_ReactivateCompletion, 0, &otherTryFinally);
+
+        if (exitContextNeeded(comp)) {
+            OpValue ourNode = OpValue::immNode(comp->functionBody());
+            CodeGen::emitOp(comp, Op_ReactivateCompletionDebug, 0, &otherTryFinally, &ourNode);
+        } else {
+            CodeGen::emitOp(comp, Op_ReactivateCompletion, 0, &otherTryFinally);
+        }
         comp->popNest();
     }
 }
@@ -1551,6 +1567,11 @@ void FunctionBodyNode::generateExecCode(CompileState* comp)
         comp->setEvalResultRegister(&evalResReg);
 
         // There is no need to initialize this as everything will be set to undefined anyway
+    } else {
+        if (comp->compileType() == Debug) {
+            OpValue ourNode = OpValue::immNode(this);
+            CodeGen::emitOp(comp, Op_EnterDebugContext, 0, &ourNode);
+        }
     }
 
     // Set unwind..
@@ -1560,13 +1581,16 @@ void FunctionBodyNode::generateExecCode(CompileState* comp)
     BlockNode::generateExecCode(comp);
 
     // Make sure we exit!
-    if (comp->codeType() != FunctionCode)
-        CodeGen::emitOp(comp, Op_ExitEval, 0, &evalResVal);
-    else
+    if (comp->codeType() != FunctionCode) {
+        CodeGen::emitOp(comp, Op_Return, 0, &evalResVal);
+    } else {
+        generateExitContextIfNeeded(comp);
         CodeGen::emitOp(comp, Op_Exit);
+    }
 
     // Unwind stuff..
     CodeGen::patchJumpToNext(comp, unwind, 0);
+    generateExitContextIfNeeded(comp);
     CodeGen::emitOp(comp, Op_PropagateException);
 }
 
