@@ -1,20 +1,21 @@
- /*  This file is part of the KDE project
-    Copyright (C) 2005,2006 Kevin Ottens <ervin@kde.org>
+/*
+    Copyright 2005,2006 Kevin Ottens <ervin@kde.org>
 
     This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Library General Public
-    License version 2 as published by the Free Software Foundation.
+    modify it under the terms of the GNU Lesser General Public
+    License as published by the Free Software Foundation; either
+    version 2.1 of the License, or (at your option) version 3, or any
+    later version accepted by the membership of KDE e.V. (or its
+    successor approved by the membership of KDE e.V.), which shall
+    act as a proxy defined in Section 6 of version 3 of the license.
 
     This library is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Library General Public License for more details.
-
-    You should have received a copy of the GNU Library General Public License
-    along with this library; see the file COPYING.LIB.  If not, write to
-    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-    Boston, MA 02110-1301, USA.
+    Lesser General Public License for more details.
 
+    You should have received a copy of the GNU Lesser General Public
+    License along with this library. If not, see <http://www.gnu.org/licenses/>.
 */
 
 #include "wmimanager.h"
@@ -34,21 +35,41 @@ public:
     {
     }
 
-    ~WmiManagerPrivate() 
+    ~WmiManagerPrivate()
     {
     }
-    
+
     WmiQuery::ItemList sendQuery( const QString &wql )
     {
 		return WmiQuery::instance().sendQuery( wql );
     }
+
+    QSet<Solid::DeviceInterface::Type> supportedInterfaces;
 };
 
 
 WmiManager::WmiManager(QObject *parent)
     : DeviceManager(parent),  d(new WmiManagerPrivate())
 {
-    
+    d->supportedInterfaces << Solid::DeviceInterface::GenericInterface
+                           << Solid::DeviceInterface::Processor
+                           << Solid::DeviceInterface::Block
+                           << Solid::DeviceInterface::StorageAccess
+                           << Solid::DeviceInterface::StorageDrive
+                           << Solid::DeviceInterface::OpticalDrive
+                           << Solid::DeviceInterface::StorageVolume
+                           << Solid::DeviceInterface::OpticalDisc
+                           << Solid::DeviceInterface::Camera
+                           << Solid::DeviceInterface::PortableMediaPlayer
+                           << Solid::DeviceInterface::NetworkInterface
+                           << Solid::DeviceInterface::AcAdapter
+                           << Solid::DeviceInterface::Battery
+                           << Solid::DeviceInterface::Button
+                           << Solid::DeviceInterface::AudioInterface
+                           << Solid::DeviceInterface::DvbInterface
+                           << Solid::DeviceInterface::Video
+                           << Solid::DeviceInterface::SerialInterface
+                           << Solid::DeviceInterface::SmartCardReader;
 }
 
 WmiManager::~WmiManager()
@@ -56,6 +77,16 @@ WmiManager::~WmiManager()
     delete d;
 }
 
+QString WmiManager::udiPrefix() const
+{
+    return QString(); //FIXME: We should probably use a prefix there... has to be tested on Windows
+}
+
+QSet<Solid::DeviceInterface::Type> WmiManager::supportedInterfaces() const
+{
+    return d->supportedInterfaces;
+}
+
 QStringList WmiManager::allDevices()
 {
     QStringList deviceUdiList;
@@ -78,7 +109,7 @@ bool WmiManager::deviceExists(const QString &udi)
 QStringList WmiManager::devicesFromQuery(const QString &parentUdi,
                                          Solid::DeviceInterface::Type type)
 {
-    qDebug() << parentUdi << type;
+//    qDebug() << parentUdi << type;
     if (!parentUdi.isEmpty())
     {
         QStringList result = findDeviceStringMatch("info.parent", parentUdi);
@@ -120,13 +151,13 @@ QStringList WmiManager::findDeviceStringMatch(const QString &key, const QString
     qDebug() << "has to be implemented" << key << value;
     QStringList result;
 
-    qDebug() << result;
+//    qDebug() << result;
     return result;
 }
 
-QStringList WmiManager::findDeviceByDeviceInterface(const Solid::DeviceInterface::Type &type)
+QStringList WmiManager::findDeviceByDeviceInterface(Solid::DeviceInterface::Type type)
 {
-    qDebug() << type;
+//    qDebug() << type;
     QStringList result;
     WmiQuery::ItemList list;
 
@@ -140,6 +171,7 @@ QStringList WmiManager::findDeviceByDeviceInterface(const Solid::DeviceInterface
     case Solid::DeviceInterface::Block:
         break;
     case Solid::DeviceInterface::StorageAccess:
+        result << WmiDevice::generateUDIList(type);
         break;
     case Solid::DeviceInterface::StorageDrive:
         break;
@@ -170,12 +202,13 @@ QStringList WmiManager::findDeviceByDeviceInterface(const Solid::DeviceInterface
         break;
     case Solid::DeviceInterface::Video:
         break;
+    case Solid::DeviceInterface::NetworkShare:
     case Solid::DeviceInterface::Unknown:
     case Solid::DeviceInterface::Last:
         break;
     }
 
-    qDebug() << result;
+//    qDebug() << result;
     return result;
 }
 
